@@ -1088,7 +1088,7 @@ static inline char *dlerror(void)
 {
   static char win_errormsg[2048];
   FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM,
-    0, GetLastError(), 0, win_errormsg, 2048, NULL);
+    0, GetLastError(),  MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US), win_errormsg, 2048, NULL);
   return win_errormsg;
 }
 #define HAVE_DLOPEN 1
